@@ -1,4 +1,6 @@
 #include "ZsubstRec.h"
+c        epifCross is changed radically.
+c        epbndry; icon is changed to get el in real*8
 c        *********************************************************
 c        *
 c        *  epgen:generate  showers until stack area becomes empty
@@ -360,6 +362,7 @@ c                  -1:  no more ptcl
 c
        record /epTrack/ aTrack
        integer icon
+       pathInB = 0.
  10    continue
 c         do 876 until (ok)
        if(Stack_pos .gt. 0) then
@@ -375,6 +378,7 @@ c                                  if IncGp=0.
 c              do business when a new component appear
           call epnewComp
           call epchckE(aTrack, icon)
+
           if(icon .ne. 0) goto 10
 
        else
@@ -456,6 +460,9 @@ c           adjust momentum; because of  energy change
             endif
 
 c            if(.not. Move.Cross .and. cTrack.p.charge .ne. 0) then
+c               we don't use above judgement; later at epCross, we use new
+c               angle due to scattering at Move.boundary. (don't use
+c               new  position due to scattering in epCross)
 c           ++++++++++
             if(cTrack.p.charge .ne. 0) then
 c                 multiple scattering, magnetic deflection
@@ -554,7 +561,8 @@ c              local. coord..
 c     *************** see if Move.Track crosses the boundary ******
 c              set Cross=t/f, if t, adjust path and set Trunc=t
       call epifCross(icon)
-      if(icon .eq. 0) then
+      icon = 0
+cc      if(icon .eq. 0) then
 c     
 c     ************** energy loss consideration ******************
 c        if E becomes <= m,  adjust path and set Trunc=t,
@@ -564,7 +572,7 @@ c        and reset Cross.
          else
             Move.dE = 0.
          endif
-      endif
+cc      endif
       end
 c     *******************************
       subroutine epCross(icon)
@@ -585,43 +593,44 @@ c                                 other values are not given here
 c
 c      save the cTrack
       aTrack = cTrack
-c       cross the boundary
+c       cross the boundary;  use  new angle at the boundary.
+c      (not cTrack.w.x etc)
       cTrack.pos.x = Move.boundary.x +
-c     *     EpsLeng* Move.Track.w.x
-     *     EpsLeng* cTrack.w.x   !+++++++++
+     *     EpsLeng* Move.Track.w.x  
 c          if EpsLeng and w.x are both small, and x is large
 c          no move may happen, avoid such case.
 c               You may give larger EpsLeng in input data.
 c
       if( cTrack.w.x .ne. 0. ) then
          n = 2
-         do while( cTrack.pos.x .eq. Move.boundary.x ) 
+         do while( cTrack.pos.x .eq. Move.boundary.x .and.
+     *             n .lt. 10 ) 
             cTrack.pos.x = Move.boundary.x +
-     *           n*EpsLeng* cTrack.w.x   
+     *           n*EpsLeng* Move.Track.w.x
             n = n * 2
          enddo
       endif
 
       cTrack.pos.y = Move.boundary.y +
-c     *     EpsLeng* Move.Track.w.y 
-     *     EpsLeng* cTrack.w.y   !+++++++
+     *     EpsLeng* Move.Track.w.y
       if( cTrack.w.y .ne. 0.) then
          n = 2
-         do while( cTrack.pos.y .eq. Move.boundary.y ) 
+         do while( cTrack.pos.y .eq. Move.boundary.y
+     *             .and. n .lt. 10 ) 
             cTrack.pos.y = Move.boundary.y +
-     *           n*EpsLeng* cTrack.w.y   
+     *           n*EpsLeng*  Move.Track.w.y
             n = n * 2
          enddo
       endif
 
       cTrack.pos.z = Move.boundary.z +
-c     *     EpsLeng* Move.Track.w.z
-     *     EpsLeng* cTrack.w.z   !++++++++++
+     *     EpsLeng* Move.Track.w.z
       if(cTrack.w.z .ne. 0. ) then
          n = 2
-         do while( cTrack.pos.z .eq. Move.boundary.z ) 
+         do while( cTrack.pos.z .eq. Move.boundary.z  .and.
+     *          n .lt. 10 )
             cTrack.pos.z = Move.boundary.z +
-     *           n*EpsLeng* cTrack.w.z
+     *           n*EpsLeng* Move.Track.w.z
             n = n * 2
          enddo
       endif
@@ -632,15 +641,19 @@ c     *     EpsLeng* Move.Track.w.z
       call epsubvec(Move.Track.w, cTrack.w)
 #endif
 
-c         new comp. number
+c         new comp. number;  new position would be, in principle,
+c         in a new component. but scattering at the boundary may
+c         cause the new pos to be in the same component.
       call eppos2cn(Cn, cTrack, cnx)
-      if(cnx .gt. Det.nct) then
+      if(Cn .eq. cnx) then
+         Move.Cross = .false.
+      elseif(cnx .gt. Det.nct) then
          info = 0   ! exiting to void
       else
          info = -cnx   ! exiting to cnx
       endif
       cTrack.cn = cnx
-      if(Det.cmp(Cn).CountIO .ge. 2) then
+      if(Det.cmp(Cn).CountIO .ge. 2 .and. Move.Cross) then
 c          user hook for  counting exiting ptcls 
 c                    exiting from Cn to cnx
          call userbd(info, aTrack, Move, Media(MediaNo))
@@ -660,7 +673,7 @@ c////////////////
       if(cnx .gt. Det.nct) then
 c          void
          icon = 1
-      else
+      elseif(Move.Cross) then
          info = Cn            ! save the current cn
 c         update coord. to local one in new  comp.
          call epl2w(Cn, cTrack.pos, postemp)
@@ -682,7 +695,7 @@ c            update comp. info (Cn, MediaNo etc are updated)
 
          if(Det.cmp(Cn).CountIO .eq. 1 .or.
      *      Det.cmp(Cn).CountIO .eq. 3 ) then
-c             count entering ptcls;  from Cn to Move.Track.cn
+c             count entering ptcls;  from info to Move.Track.cn (=Cn)
             call userbd(info, aTrack, Move, Media(MediaNo)) 
             if(Move.Abort .ne. 0) then
                if(Move.Abort .eq. 3) then
@@ -1178,138 +1191,38 @@ c             see if Move.Track crosses a boundary and set Cross = t/f
 c        If t, reset Move.Track.pos just before the boundary.
 c        set exact boundary in Move.boundary.
 c
+      logical kcon, icon
 
 
-      integer icon, cnxx, i, cond, kcon
-      logical bfound, sameComp
-      real*8 el, tempel
+      real*8 el
 
 c         see if Move.Track is in Cn-th component.
 c          coord is local coord.
-
-      call epsOne(Cn, Move.Track.pos, icon)
-
-
-      if(icon .eq. 0) then
-c          Move.Track is in the same component. But it may 
-c          entered into an inner Matreshka; examine it.
-         call epsMatreska(Cn, Move.Track.pos, cnxx)
-c ////////////////
-c         call debugpos('ifCross; matreska',Cn, 
-c     *       Move.Track.pos, cnxx)
-c///////////////
-         if(cnxx .ne. Cn) then
-c              child matreshka found, so that
-c              Move.Track is outside of the current volume
-            icon = 1
-         else
-c            then examine if the pos in some union part.
-
-            call epsPCont(Cn, Move.Track.pos, icon)
-c     /////////////////
-c            call debugpos('ifCross; epsPCont', Cn, 
-c     *          Move.Track.pos, icon)
-c /////////////
-c             icon !=0 means, pos is outside of the union
-c             part of the current volume already
-         endif
-      endif
-c                judge if in the same component
-      bfound = .false.  ! boundary is not yet found
-      sameComp = icon .eq. 0
-
-      
-      if(sameComp)  then
-c         Move.Track is in Cn-th comp. but  may
-c         crosses an inner matreska. examine it.
-c         e.g; starting point S and ending point E
-c         are in the same component but it crosses
-c         the inner matreshka; 
-c                  
-c                  S   
-c                   \
-c                    \   * *
-c                      *  Inner Matreshka
-c                     *\       *
-c                     * \      *
-c                      * \    *
-c                        *\* *
-c                          \
-c                           E       
-
-         if(Det.cmp(Cn).NMatreska .gt. 0) then
-c            current comp.  is Matreska  so that
-c            we have to examine inner  comp.
-c            length to the new position.
-            el = sqrt((Move.Track.pos.x-cTrack.pos.x)**2 +
-     *                (Move.Track.pos.y-cTrack.pos.y)**2 +
-     *                (Move.Track.pos.z-cTrack.pos.z)**2 )
-            do i = 1, Det.cmp(Cn).NMatreska
-c               length to the boundary of the Det.cmp(Cn).Contains(i)-th
-c               component which is i-th matreshka of Cn-th comp.
-c(((((((((((
-c               call epbndry2(Det.cmp(Cn).Contains(i), tempel, icon)
-               call epbndry2(CnArea(Det.cmp(Cn).Contains+i),
-     *          tempel, icon)
-c))))))))))))))
-c          //////////
-c               call debugpos('ifCross; epbndry2',Cn, 
-c     *          Move.Track.pos, icon)
-c//////////////////
-c                  icon = 1; pos is   outside of the examined comp.
-c                    and the length is tempel.
-               if(icon .eq. 1 .and. tempel .lt. el) then
-c                    really crosses
-                  el = tempel
-                  bfound = .true.
-                  sameComp = .false.
-               endif
-            enddo
-         endif
-      endif
-      if(sameComp) then
-         Move.Cross =  .false.
+      call epbndry(Move.boundary, el, icon)
+c       icon =1 indicates something wrong. but 
+c       it should have been corrected safely so we don't
+c       care.
+      if(Move.dl .lt. el) then
+         kcon = 0
+         Move.Cross = .false.
       else
+         kcon = 1
          Move.Cross = .true.
          Move.Trunc = .true.
-c          current component to another one; get boundary
-         if(.not. bfound) then
-c                    we have to find boundary; it may be
-c                the one  with some matreska inside.
-            call epbndry(Move.boundary, cond)
-c////////////////////
-c            call debugpos('ifCross; epbndry', Cn, 
-c     *       Move.boundary, cond)
-c///////////////
-            if(cond .eq. 2) then
-               call cerrorMsg('boundary not found; strange',0) 
-            elseif(cond .eq. 1) then
-               kcon = 1
-               return  !  emergency
-            endif
-         else
-            Move.boundary.x = cTrack.pos.x + el* cTrack.w.x
-            Move.boundary.y = cTrack.pos.y + el* cTrack.w.y
-            Move.boundary.z = cTrack.pos.z + el* cTrack.w.z
-         endif 
          Move.Track.pos.x = Move.boundary.x -
-c     *             EpsLeng* Move.Track.w.x
      *             EpsLeng* cTrack.w.x
          Move.Track.pos.y = Move.boundary.y -
-c     *             EpsLeng* Move.Track.w.y
      *             EpsLeng* cTrack.w.y
          Move.Track.pos.z = Move.boundary.z -
-c     *             EpsLeng* Move.Track.w.z
      *             EpsLeng* cTrack.w.z
-         Move.dl = sqrt(
-     *          (Move.Track.pos.x-cTrack.pos.x)**2 +
-     *          (Move.Track.pos.y-cTrack.pos.y)**2 +
-     *          (Move.Track.pos.z-cTrack.pos.z)**2 )
-
+c         Move.dl = sqrt(
+c     *          (Move.Track.pos.x-cTrack.pos.x)**2 +
+c     *          (Move.Track.pos.y-cTrack.pos.y)**2 +
+c     *          (Move.Track.pos.z-cTrack.pos.z)**2 )
+         Move.dl = el - EpsLeng
          Move.dt = Move.dl/Media(MediaNo).X0
          Move.dx = Move.dl/Media(MediaNo).gtocm 
       endif
-      kcon  = 0
       end
 c
       subroutine epchckE(aTrack, icon)
